package pintorj2d;

import java.awt.Color;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;
import java.util.ArrayList;
import javax.swing.JPanel;
import javax.swing.Timer;
import util.FormaGeometrica;

/**
 * Instituto Federal de Santa Catarina - IFSC - Campus São José
 * Disciplina: Programação Orientada a Objetos
 *
 * Esse painel irá representar a área de desenho que permite adicionar formas
 * geométricas.
 *
 * Algumas dicas para brincar com Java 2D
 *
 * http://java.sun.com/docs/books/tutorial/2d/index.html
 * http://java.sun.com/docs/books/tutorial/2d/geometry/strokeandfill.html
 *
 * @author Emerson Ribeiro de Mello
 */
public class Area extends JPanel implements ActionListener {

    private Timer timer;
    private int origemx;
    private int origemy;
    private int destinox;
    private int destinoy;
    private JanelaPrincipal pai;
    public ArrayList<FormaGeometrica> formasDesenhadas;

    public Area(JanelaPrincipal jp) {

        this.pai = jp;

        // onde iremos armazenar as formas desenhadas na área de desenho
        this.formasDesenhadas = new ArrayList<FormaGeometrica>();


        // adicionando tratadores de eventos do mouse
        this.addMouseListener(new TratarEventosBotaoMouse());
        this.addMouseMotionListener(new TratarEventosMovimentoMouse());

        // alterando algumas propriedades do objeto
        this.setFocusable(true); // pode assumir foco
        this.setBackground(Color.WHITE); // alterando a cor do fundo
        this.setDoubleBuffered(true); // tornando o redesenho mais rapido

        /* Irá atualizar a tela a cada 100 milisegundos */
        this.timer = new Timer(100, this);
        this.timer.start();

        this.zerarCoordenadas();
    }

    /**
     * Método que será invocado através do repaint();
     * @param g
     */
    @Override
    public void paint(Graphics g) {
        super.paint(g);

        Graphics2D g2d = (Graphics2D) g;

        g2d.setColor(Color.blue);

        // ativando o antialiasing
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

        // imprimindo na tela as formas que já foram desenhadas
        this.desenharFormasAnteriores(g2d);


        // imprimindo na tela a forma que está sendo desenhada no exato momento
        if (destinox != -1) {

            int x, la, y, al;


            // para descobrir se o x1 é menor ou maior que x2 para entao
            // calcular a largura
            if (origemx < destinox) {
                x = origemx;
                la = destinox - origemx;
            } else {
                x = destinox;
                la = origemx - destinox;
            }

            // para descobrir se o y1 é menor ou maior que y2 para entao
            // calcular a altura
            if (origemy < destinoy) {
                y = origemy;
                al = destinoy - origemy;
            } else {
                y = destinoy;
                al = origemy - destinoy;
            }

            // será um retângulo ou uma elipse?
            if (pai.formaGeometrica.equals("retangulo")) {

                g2d.drawRect(x, y, la, al);

                /* Outra forma de desenhar um retângulo preenchido
                 *
                 * g2d.draw(new Rectangle2D.Double(x, y, la, al));
                 *
                 * O pacote java.awt.geom. possui algumas formas interessantes
                 *
                 */
            } else if (pai.formaGeometrica.equals("elipse")) {
                // colocando uma cor de fundo para a forma
                // essa cor só será usada na forma que acabara de ser desenhada
                // tem que pensar em uma forma para permitir que as formas já
                // desenhadas mantenham a cor
                g2d.setPaint(new Color(0, 255, 0, 125));

                /*
                 * a forma abaixo será preenchida com uma cor de acordo com a
                 * tabela RGB e o último parâmetro ALPHA para indicar o nível de
                 * transparência.
                 *
                 * Poderíamos usar somente cores sólidas, assim o setPain seria:
                 *
                 * g2d.setPaint(Color.green);
                 *
                 * Poderíamos também preencher com gradiente
                 *
                 * g2d.setPaint(new GradientPaint(0, 0, new Color(0, 150, 0), la, x,
                new Color(0, 150, 0, 0), true));
                 *
                 *
                 */

                g2d.fillOval(x, y, la, al);

                /* Outra forma de desenhar uma elipse preenchida
                 * 
                 * g2d.fill(new Ellipse2D.Double(x, y, la, al));
                 *
                 * O pacote java.awt.geom. possui algumas formas interessantes
                 */



            }


            /* outras formas
             *
             * g2d.fillRect(x, y, la, al); // retangulo preenchido
             * g2d.drawLine(origemx, origemy, destinox, destinoy);
             * g2d.drawRoundRect(x, y, la, al,10,20);
             * g2d.drawOval(x, y, la, al);
             * g2d.drawArc(x,y,w,h,sdeg,ndeg)
             * g2d.drawPolygon(x[],y[],numPoints)
             * 
             */

        }

    }

    /**
     * Faz com que as coordenadas de clique sejam zeradas com valores não possíveis
     * na área de desenho.
     */
    public void zerarCoordenadas() {
        origemx = -1;
        origemy = -1;
        destinox = -1;
        destinoy = -1;
    }

    /**
     * Será executado cada N milisegundos. Ou seja, a tela será redesenhada.
     *
     * A atualização constante da tela é necessária para permitir o efeito
     * instantâneo de clicar e arrastar para desenhar uma forma
     *
     * @param e
     */
    public void actionPerformed(ActionEvent e) {
        // pintando a tela novamente
        repaint();
    }

    /**
     * Irá percorrer todas as formas já desenhadas e armazenadas no ArrayList
     * para desenhá-las na tela
     * @param g2d - Tela onde será desenhada as imagens
     */
    private void desenharFormasAnteriores(Graphics2D g2d) {
			// fazendo uso do for de uma forma diferente para percorrer um ArrayList
			// Sintaxe: for (NomeDaClasse variavelLocalAoFOR : nomedoArrayList)
        for (FormaGeometrica forma : formasDesenhadas) {

            if (forma.tipo.equals("retangulo")) {
                g2d.drawRect(forma.x, forma.y, forma.largura, forma.altura);
            } else if (forma.tipo.equals("elipse")) {
                g2d.drawOval(forma.x, forma.y, forma.largura, forma.altura);
            }

        }
    }

    /**
     * Apagando todas as formas amarzenadas e limpando a tela
     */
    public void limparImagem() {

        this.formasDesenhadas.clear();
        this.zerarCoordenadas();
    }

    /**
     * Classe interna e privada para tratar eventos de botão do mouse
     *
     * Uma classe interna consegue acessar todos os atributos (mesmo os privados)
     * da classe mais externa que a contém.
     *
     */
    private class TratarEventosBotaoMouse extends MouseAdapter {

        /**
         * Quando o botão do mouse for pressionado
         * @param e evento do mouse
         */
        @Override
        public void mousePressed(MouseEvent e) {
            destinox = -1;
            destinoy = -1;
            origemx = e.getX();
            origemy = e.getY();

            // Quando se está desenhando algo, então a tela será atualizada a
            // cada 5 milisegundos
            // acessando atributo timer da classe externa
            timer.setDelay(5);
        }

        /**
         * Quando o botão do mouse for liberado
         * @param e evento do mouse
         */
        @Override
        public void mouseReleased(MouseEvent e) {
            destinox = e.getX();
            destinoy = e.getY();

            // Quando não se está desenhando algo, então a tela será atualizada a
            // cada 100 milisegundos. Isso faz com que nosso editor não fique
            // consumindo muito o CPU.
            // acessando atributo timer da classe externa
            timer.setDelay(100);


            int x, la, y, al;
            // para descobrir se o x1 é menor ou maior que x2 para entao
            // calcular a largura
            if (origemx < destinox) {
                x = origemx;
                la = destinox - origemx;
            } else {
                x = destinox;
                la = origemx - destinox;
            }

            // para descobrir se o y1 é menor ou maior que y2 para entao
            // calcular a altura
            if (origemy < destinoy) {
                y = origemy;
                al = destinoy - origemy;
            } else {
                y = destinoy;
                al = origemy - destinoy;
            }
            // guardando a forma desenhada
            // acessando atributo formasDesenhadas da classe externa
            // e acessando o atributo formGeometrica da classe JanelaPrincipal
            formasDesenhadas.add(new FormaGeometrica(x, y, la, al, pai.formaGeometrica));
        }
    }

    /**
     * Classe interna e privada para tratar eventos de movimento do mouse
     *
     * Uma classe interna consegue acessar todos os atributos (mesmo os privados)
     * da classe mais externa que a contém.
     * 
     */
    private class TratarEventosMovimentoMouse extends MouseAdapter {

        /**
         * Quando o mouse for movimentado sem pressionar o botão
         * @param e evento do mouse
         */
        @Override
        public void mouseMoved(MouseEvent e) {
            // atualizando os labels que representam as coordenadas na interface
            // do usuário
            // acessando os atributos jLabel1 e jLabel2 da classe JanelaPrincipal
            pai.jLabel1.setText("" + e.getX());
            pai.jLabel2.setText("" + e.getY());
        }

        /**
         * Quando o mouse for movimentado pressionando o botão
         * @param e evento do mouse
         */
        @Override
        public void mouseDragged(MouseEvent e) {
            destinox = e.getX();
            destinoy = e.getY();

            // atualizando os labels que representam as coordenadas na interface
            // do usuário
            // acessando os atributos jLabel1 e jLabel2 da classe JanelaPrincipal
            pai.jLabel1.setText("" + e.getX());
            pai.jLabel2.setText("" + e.getY());
        }
    }
}
